pub fn aggregate_context_snapshot_channel() -> (AggregateContextSnapshotHandle, AggregateContextSnapshotReceiver)Expand description
Creates a retained-context snapshot handle and its owner-side receiver.
The receiver belongs to the aggregate transform, and is supplied to it through
AggregateConfiguration::context_snapshot_receiver. The handle belongs to whoever requests snapshots, and can be
cloned to let several callers request them.